-- Credits for Addon design belongs to Rabbit
-- this mod is based on NRT.
-- License: GPLv2

local sRT = LibStub("AceAddon-3.0"):NewAddon("sRT", "AceEvent-3.0")
local LibQTip = LibStub('LibQTip-1.0')
local icon = LibStub("LibDBIcon-1.0")

local tooltip
local LDB_ANCHOR

local DEBUG = nil
local playerName = UnitName("player")
local lastBossName = nil
local isInGroup = nil
local sRTDB_Default = {
  Loot = { },
  Boss = { },
  Groups = 5,
  Minimap = { hide = nil },
  TrackingEnabled = true,
  AnnounceLoot = nil,
  AttachTrashToBoss = nil,
  Formatter = "Plain text",
}

local lootSelf = (LOOT_ITEM_SELF):gsub("%%s", "(.+)")
local lootSelfMultiple = (LOOT_ITEM_SELF_MULTIPLE):gsub("%%sx%%d", "(.+)x(%%d+)")
local loot = (LOOT_ITEM):gsub("%%s", "(.+)")
local lootMultiple = (LOOT_ITEM_MULTIPLE):gsub("%%s receives loot: %%sx%%d", "(.+) receives loot: (.+)x(%%d+)")
local ifDateFormat = "%Y-%m-%d %H:%M:%S"

local worldbosses = { }
local killedBosses = { }
local outputFormatters = { }
local formatters = { }

------------------------------------
---- Item and Boss customization ---
------------------------------------

local ignoredItems = {
	[36919] = true, -- Cardinal Ruby
	[36925] = true, -- Majestic Zircon
	[36922] = true, -- King's Amber
	[36928] = true, -- Dreadstone
	[36934] = true, -- Eye of Zul
	[36931] = true, -- Ametrine

	[34057] = true, -- Abyss Crystal
}

local ignoredBosses = {
	"Drek'Thar",
	"Vanndar Stormpike",
	"Captain Galvangar",
	"Captain Balinda Stonehearth",
	"Madrigosa",
	"Tenebron",
	"Shadron",
	"Vesperon",
	"Baron Rivendare",
	"Lady Blaumeux",
	"Thane Korth'azz",
	"Sanctum Sentry",
	"Feral Defender",
	"Right Arm",
	"Left Arm",
	"Leviathan Mk II",
	"VX-001",
	"Aerial Command Unit",
	"Elder Stonebark",
	"Elder Ironbranch",
	"Elder Brightleaf",
	"Gormok the Impaler",
	"Acidmaw",
	"Dreadscale",
	"Fjola Lightbane",
	"Prince Valanar",
	"Prince Taldaram",
	"General Zarithrian",
	"Baltharus the Warborn",
	"Saviana Ragefire",
	"Raging Spirit",
	"Val'kyr Shadowguard",
	"Ice Sphere",
}

local addedBosses = {
	"Lady Deathwhisper",
	"Professor Putricide",
	"Sindragosa",
	"Halion",
}
	
local bossDefeatedYells = {
	["Mimiron"] = "It would appear that I've made a slight miscalculation. I allowed my mind to be corrupted by the fiend in the prison, overriding my primary directive. All systems seem to be functional now. Clear.",
	["Thorim"] = "Stay your arms! I yield!",
	["Hodir"] = "I... I am released from his grasp... at last.",
	["Freya"] = "His hold on me dissipates. I can see clearly once more. Thank you, heroes.",
	["High Overlord Saurfang"] = "The Alliance falter. Onward to the Lich King!",
	["Muradin Bronzebeard"] = "Don't say I didn't warn ya, scoundrels! Onward, brothers and sisters!",
	["Valithria Dreamwalker"] = "I AM RENEWED! Ysera grant me the favor to lay these foul creatures to rest!",
}

----------------------------------------------
------------- Font definitions ---------------
----------------------------------------------

local ssTitleFont = CreateFont("ssTitleFont")
ssTitleFont:SetTextColor(1,0.823529,0)
ssTitleFont:SetFont(GameTooltipText:GetFont(), 12)

local ssHeaderFont = CreateFont("ssHeaderFont")
ssHeaderFont:SetTextColor(1,0.823529,0)
ssHeaderFont:SetFont(GameTooltipHeaderText:GetFont(), 10)

local ssRegFont = CreateFont("ssRegFont")
ssRegFont:SetTextColor(1,0.823529,0)
ssRegFont:SetFont(GameTooltipText:GetFont(), 10)

local dataobj = LibStub("LibDataBroker-1.1"):NewDataObject("sRT",
{
	type	= "data source",
	icon	= "Interface\\Icons\\INV_Crate_01.png",
	label	= "sRT",
	text	= "sRT"
})


----------------------------
-----  Tooltip events  -----
----------------------------

local function Raid_OnMouseUp(frame, info, button)

	if button == "LeftButton" then
		if IsShiftKeyDown() then
			table.remove(sRTDB.Boss, info)
			dataobj.OnEnter(LDB_ANCHOR)
		elseif IsAltKeyDown() then
			for i = 1, GetNumGroupMembers() do
				local n, _, subgroup = GetRaidRosterInfo(i)
				if subgroup <= sRTDB.Groups and (not sRT:IsInTable(sRTDB.Boss[info].attendants, n)) then
					sRTDB.Boss[info].attendants[#sRTDB.Boss[info].attendants + 1] = n
				end
			end
			sRT:PrintMessage("Raid " .. sRTDB.Boss[info].name .. " has been updated")
		else
			sRT:Export(info)
		end
	end
end

local function Loot_OnMouseUp(frame, info, button)
	
	local loot = sRTDB.Loot[info]

	if button == "LeftButton" then
		if IsShiftKeyDown() then
			table.remove(sRTDB.Loot, info)
			dataobj.OnEnter(LDB_ANCHOR)
		elseif IsAltKeyDown() then
			if ChatEdit_GetActiveWindow() then
				ChatEdit_InsertLink(loot.item)
			end
		elseif IsControlKeyDown() then
			DressUpItemLink(loot.item)
		else
			SetItemRef(loot.item);
		end
	end

end


function dataobj:OnClick(button)
	if button == "LeftButton" then
		if IsAltKeyDown() then
			sRT:TakeAttendance()
			dataobj.OnEnter(LDB_ANCHOR)
		end
	elseif button == "RightButton" then
		InterfaceOptionsFrame_OpenToCategory("sRT")
	end
end

local function EditNote(frame, i, button)

	StaticPopupDialogs["sRTNote"] = {
		text = "",
		button1 = "Set",
		button2 = "Cancel",
		whileDead = 1,
		hideOnEscape = 1,
		timeout = 0,
		OnShow = function(self)
			self.editBox:SetText(sRTDB.Loot[i].note or "")
		end,
		OnHide = function(self)
			self.editBox:SetText("")
		end,
		EditBoxOnEnterPressed = function(self)
			sRTDB.Loot[i].note = self:GetText()
			self:GetParent():Hide()
		end,
		EditBoxOnEscapePressed = function(self)
			self:GetParent():Hide()
		end,
		OnAccept = function(self)
			sRTDB.Loot[i].note = self.editBox:GetText()
		end,
		hasEditBox = 1,
	}
	
	
	StaticPopup_Show("sRTNote")
end

local function EditPlayer(frame, i, button)

	StaticPopupDialogs["sRTPlayer"] = {
		text = "",
		button1 = "Set",
		button2 = "Cancel",
		whileDead = 1,
		hideOnEscape = 1,
		timeout = 0,
		OnShow = function(self)
			self.editBox:SetText(sRTDB.Loot[i].player or "")
		end,
		OnHide = function(self)
			self.editBox:SetText("")
		end,
		EditBoxOnEnterPressed = function(self)
			sRTDB.Loot[i].player = self:GetText()
			self:GetParent():Hide()
		end,
		EditBoxOnEscapePressed = function(self)
			self:GetParent():Hide()
		end,
		OnAccept = function(self)
			sRTDB.Loot[i].player = self.editBox:GetText()
		end,
		hasEditBox = 1,
	}
	
	
	StaticPopup_Show("sRTPlayer")
end

local function EditFrom(frame, i, button)

	StaticPopupDialogs["sRTFrom"] = {
		text = "",
		button1 = "Set",
		button2 = "Cancel",
		whileDead = 1,
		hideOnEscape = 1,
		timeout = 0,
		OnShow = function(self)
			self.editBox:SetText(sRTDB.Loot[i].from or "")
		end,
		OnHide = function(self)
			self.editBox:SetText("")
		end,
		EditBoxOnEnterPressed = function(self)
			sRTDB.Loot[i].from = self:GetText()
			self:GetParent():Hide()
		end,
		EditBoxOnEscapePressed = function(self)
			self:GetParent():Hide()
		end,
		OnAccept = function(self)
			sRTDB.Loot[i].from = self.editBox:GetText()
		end,
		hasEditBox = 1,
		editBoxWidth = 300,
	}
	
	
	StaticPopup_Show("sRTFrom")
end

----------------------------
----- Tooltip --------------
----------------------------

function dataobj.OnEnter(self)
	LDB_ANCHOR = self

	if LibQTip:IsAcquired("sRT") then
		LibQTip:Release(tooltip)
		tooltip = nil
	end

	tooltip = LibQTip:Acquire("sRT", 4, "LEFT", "CENTER", "LEFT", "LEFT")

	tooltip:SetBackdropColor(0,0,0,1)

	tooltip:SetHeaderFont(ssHeaderFont)
	tooltip:SetFont(ssRegFont)

	tooltip:SmartAnchorTo(self)
	tooltip:SetAutoHideDelay(0.25, self)

	local line = tooltip:AddLine()
	tooltip:SetCell(line, 1, "sRT", ssTitleFont, "CENTER", 0)

	-------------------------
	--------  Raid List  ----
	-------------------------

	if #sRTDB.Boss > 0 then
		line = tooltip:AddLine()
		tooltip:SetCell(line, 1, "|cffffffff" .. "Raid List" .. "|r", "LEFT", 3)

		line = tooltip:AddHeader()
		line = tooltip:SetCell(line, 1, "Boss")
		line = tooltip:SetCell(line, 2, "Zone")
		line = tooltip:SetCell(line, 3, "Time")

		tooltip:AddSeparator()

		for i = 1, #sRTDB.Boss do
			local raid = sRTDB.Boss[i]

			line = tooltip:AddLine()
			line = tooltip:SetCell(line, 1, raid.name)
			line = tooltip:SetCell(line, 2, raid.zone)
			line = tooltip:SetCell(line, 3, date(ifDateFormat, raid.date))

			tooltip:SetLineScript(line, "OnMouseUp", Raid_OnMouseUp,  i)
		end
		
	tooltip:AddLine(" ")
	end
		
	-------------------------
	--------  Loot List  ----
	-------------------------

	if #sRTDB.Loot > 0 then
		line = tooltip:AddLine()
		tooltip:SetCell(line, 1, "|cffffffff" .. "Loot" .. "|r", "LEFT", 3)

		line = tooltip:AddHeader()
		line = tooltip:SetCell(line, 1, "Item")
		line = tooltip:SetCell(line, 2, "Player")
		line = tooltip:SetCell(line, 3, "Boss")
		line = tooltip:SetCell(line, 4, "Note")

		tooltip:AddSeparator()

		for i = 1, #sRTDB.Loot do
			local item = sRTDB.Loot[i]

			line = tooltip:AddLine()
			line = tooltip:SetCell(line, 1, item.item)
			line = tooltip:SetCell(line, 2, item.player)
			line = tooltip:SetCell(line, 3, item.from)
			if item.note then
				line = tooltip:SetCell(line, 4, item.note)
			else
				line = tooltip:SetCell(line, 4, "")
			end

			tooltip:SetCellScript(line, 1, "OnMouseUp", Loot_OnMouseUp, i)
			tooltip:SetCellScript(line, 2, "OnMouseUp", EditPlayer, i)
			tooltip:SetCellScript(line, 3, "OnMouseUp", EditFrom, i)
			tooltip:SetCellScript(line, 4, "OnMouseUp", EditNote, i)
		end
	end
		
	tooltip:AddLine(" ")


	line = tooltip:AddLine()
	tooltip:SetCell(line, 1, "|cffeda55fClick|r raid to export.                             |cffeda55fShift-Click|r raid to delete.", "LEFT", 0)

	line = tooltip:AddLine()
	tooltip:SetCell(line, 1, "|cffeda55fAlt-Click|r raid to add your current raid members to raid you've clicked.", "LEFT", 0)
	
	line = tooltip:AddLine()
	tooltip:SetCell(line, 1, "|cffeda55fShift-Click|r loot to delete.                            |cffeda55fAlt-Click|r loot to link.", "LEFT", 0)
	
	line = tooltip:AddLine()
	tooltip:SetCell(line, 1, "|cffeda55fAlt-Click|r icon to take attendance.     |cffeda55fClick|r player, boss or note to edit.", "LEFT", 0)

	tooltip:UpdateScrolling()
	tooltip:Show()
end


-----------------------------------
-- Set up configuration window ----
-----------------------------------

local options = {
    name = "sRT",
    desc = "Stalker's RaidTracker",
    handler = sRT,
    type = "group",
    args = {
        general = {
            name = 'General',
            type = 'group',
            childGroups = "tab",
            args = {
                hdrMain = {
                    name = "Addon options",
                    order = 1,
                    type = "header",
                },
                cbEnabled = {
                    name = "Enable tracking",
                    desc = "While disabled you can still take attendance manually.\n\nYou need to reload interface for changes to take effect.",
                    order = 2,
                    type = "toggle",
                    get = function() return sRTDB.TrackingEnabled end,
					set = function(_, s)
						sRTDB.TrackingEnabled = s
					end,
                },
				cbMinimapIcon = {
                    name = "Show minimap icon",
                    desc = "Enable to show minimap icon.",
                    order = 3,
                    type = "toggle",
                    get = function() return (not sRTDB.Minimap.hide) end,
					set = function(_, s)
						sRTDB.Minimap.hide = not s
						if s then
							icon:Show("sRTLDB")
						else
							icon:Hide("sRTLDB")
						end
					end,
                },
				hdrTracking = {
                    name = "Tracking options",
                    order = 4,
                    type = "header",
                },
				cbAnnounceLoot = {
                    name = "Announce loot to guild",
                    desc = "Announce to guild chat when someone loots an epic item.",
                    order = 5,
                    type = "toggle",
                    get = function() return sRTDB.AnnounceLoot end,
					set = function(_, s)
						sRTDB.AnnounceLoot = s
					end,
                },
				cbAttachTrashToBoss = {
                    name = "Attach trash drops to last boss",
                    desc = "Attach all trash epic drops except ignored items to the last boss killed instead of the zone.\n\nIf no boss has been killed yet, it will still be attached to the zone.",
                    order = 5,
                    type = "toggle",
					width = "full",
                    get = function() return sRTDB.AttachTrashToBoss end,
					set = function(_, s)
						sRTDB.AttachTrashToBoss = s
						if not s then
							if isInGroup then
								sRT:RegisterEvent("ZONE_CHANGED_INDOORS")
							else
								sRT:UnregisterEvent("ZONE_CHANGED_INDOORS")
							end
						end
					end,
                },
				lbGroups = {
                    name = "Included groups",
                    desc = "Only include people in the groups up to the number set here. Defaults to 5, which means that only people in group 1-5 will be part of the default attendance snapshot.",
                    order = 6,
                    type = "select",
					style = "dropdown",
					values = {	[1] = "1",
								[2] = "2",
								[3] = "3",
								[4] = "4",
								[5] = "5",
								[6] = "6",
								[7] = "7",
								[8] = "8",
							},
					width = "half",
                    get = function() return sRTDB.Groups end,
					set = function(_, s)
						sRTDB.Groups = s
					end,
                },
				lbFormatters = {
                    name = "Output format",
                    desc = "Choose the output format for attendance.",
                    order = 7,
                    type = "select",
					style = "dropdown",
					values = function() return outputFormatters end,
                    get = function() return sRTDB.Formatter end,
					set = function(_, s) sRTDB.Formatter = s end,
                },
            },
        },

    }
}

local function debug(t)
	if DEBUG then print("[DEBUG] " .. t) end
end

function sRT:PrintMessage(t)
	print("|cFFFF9900sRT|r " .. t)
end

function sRT:IsInTable(table, word)
	for k,v in pairs(table) do
		if v == word then
			return true
		end
	end
	return nil
end

local function getGameTimeAsUnix()
	local h, m = GetGameTime()
	local n = date("*t")
	n.hour = h
	n.min = m
	return time(n)
end

function bossKilled(npc)
	--debug(npc .. " killed. Ignored:" .. tostring(sRT:IsInTable(ignoredBosses, npc)) .. " whitelist:" .. tostring(sRT:IsInTable(addedBosses, npc)))
	if not npc or sRT:IsInTable(ignoredBosses, npc) then return end
	if worldbosses[npc] or sRT:IsInTable(addedBosses, npc) or bossDefeatedYells[npc] then

		lastBossName = npc
		sRT:PrintMessage(npc .. " killed. Taking attendance now...")

		if not sRTDB.AttachTrashToBoss then
			sRT:PrintMessage("Please wait in boss room while loot is being distributed")
		end
		sRT:TakeAttendance()
	end
end

function sRT:TakeAttendance()

	local newTrackset = {
		name = lastBossName or GetRealZoneText(),
		zone = GetRealZoneText(),
		attendants = {},
		date = getGameTimeAsUnix(),
	}
	for i = 1, GetNumGroupMembers() do
		local n, _, subgroup = GetRaidRosterInfo(i)
		if subgroup <= sRTDB.Groups then
			newTrackset.attendants[#newTrackset.attendants + 1] = n
		end
	end
	table.insert(sRTDB.Boss, newTrackset)
end


function sRT:RegisterOutputFormatter(name, formatter)
	if type(name) ~= "string" or type(formatter) ~= "function" then
		error(("Invalid arguments to :RegisterOutputFormatter, tried registering a formatter without a proper name (%q) or function (%q)."):format(type(name), type(formatter)), 2)
	end
	if formatters[name] then
		error(("Invalid argument to :RegisterOutputFormatter, there's already an output formatter named %q registered."):format(name), 2)
	end
	--table.insert(outputFormatters, name)
	outputFormatters[name] = name
	formatters[name] = formatter
end



---------------------------
-- Event handling functions
---------------------------

function sRT:OnInitialize()

	if (not sRTDB) then
		sRTDB = sRTDB_Default;
		sRT:PrintMessage("database not found. Generating...");
	end


	if not sRTDB.Groups then
		sRTDB.Groups = 5
	end
	if not sRTDB.Formatter or sRTDB.Formatter == "Simple" then
		sRTDB.Formatter = "Plain text"
		sRT:PrintMessage("New output formats are aviable");
	end


	if sRTDB.TrackingEnabled then
		self:RegisterEvent("GROUP_ROSTER_UPDATE")
	end

	-- Register config options
	LibStub("AceConfig-3.0"):RegisterOptionsTable("sRT", options)
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions("sRT", "sRT", nil, "general")

	icon:Register("sRTLDB", dataobj, sRTDB.Minimap)

	sRT:PrintMessage("Loaded");

end

function sRT:CHAT_MSG_LOOT(_, msg)
	local player, item, quantity = playerName, nil, 1
	if msg:match(lootSelf) then
		item = msg:match(lootSelf)
	elseif msg:match(loot) then
		player, item = msg:match(loot)
	elseif msg:match(lootSelfMultiple) then
		item, quantity = msg:match(lootSelfMultiple)
	elseif msg:match(lootMultiple) then
		player, item, quantity = msg:match(lootMultiple)
	end
	if type(item) == "string" and type(player) == "string" then
		local itemName, itemLink, itemRarity = GetItemInfo(item)

		if not itemRarity or itemRarity < 4 then return end

		-- Find the item ID to make sure it's not a ignored item
		local itemId = select(3, itemLink:find("item:(%d+):"))
		if not itemId then return end
		itemId = tonumber(itemId:trim())
		if type(itemId) ~= "number" or ignoredItems[itemId] then return end
		if type(itemName) == "string" and type(itemLink) == "string" and type(itemRarity) == "number" then

		local lootedFrom = nil
		if lastBossName then
			lootedFrom = lastBossName
		else
			lootedFrom = GetRealZoneText()
		end

		if sRTDB.AnnounceLoot then
			SendChatMessage(("%s looted %s (%s)!"):format(player, itemLink, lootedFrom), "GUILD")
		end

		table.insert(sRTDB.Loot, {
			player = player,
			item = itemLink,
			itemId = itemId,
			from = lootedFrom,
			date = getGameTimeAsUnix(),
		})
		end
	end
end

function sRT:CHAT_MSG_MONSTER_YELL(_, yell, npc)
	if not bossDefeatedYells[npc] or yell ~= bossDefeatedYells[npc] then return end
	bossKilled(npc)
end

function sRT:COMBAT_LOG_EVENT_UNFILTERED()
	local timestamp, subevent, _, sourceGUID, sourceName, sourceFlags, sourceRaidFlags, destGUID, destName, destFlags, destRaidFlags = CombatLogGetCurrentEventInfo()
	if subevent ~= "UNIT_DIED" then return end
	bossKilled(destName)
end

function sRT:PLAYER_TARGET_CHANGED()
	local class = UnitClassification("target")
	local name = UnitName("target")
	if class and class == "worldboss" then
		worldbosses[name] = true
		debug("boss " .. name .. " saved to temp database")
	end
end

function sRT:GROUP_ROSTER_UPDATE()
	local num = GetNumGroupMembers()
	if not isInGroup and num > 0 then
		isInGroup = true
		sRT:PrintMessage("Tracking boss kills and loot.")
		self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
		self:RegisterEvent("CHAT_MSG_LOOT")
		self:RegisterEvent("PLAYER_TARGET_CHANGED")
		self:RegisterEvent("CHAT_MSG_MONSTER_YELL")
		if not sRTDB.AttachTrashToBoss then self:RegisterEvent("ZONE_CHANGED_INDOORS") end
	elseif isInGroup and num <= 0 then
		isInGroup = nil
		sRT:PrintMessage("Tracking disabled.")
		self:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
		self:UnregisterEvent("CHAT_MSG_LOOT")
		self:UnregisterEvent("PLAYER_TARGET_CHANGED")
		self:UnregisterEvent("CHAT_MSG_MONSTER_YELL")
		self:UnregisterEvent("ZONE_CHANGED_INDOORS")
	end

end

function sRT:ZONE_CHANGED_INDOORS()
	if lastBossName then
		sRT:PrintMessage("All items assigned from now on will be considered loot from trash.")
		lastBossName = nil
	end
end

------------------------------
------- Export ---------------
------------------------------

function sRT:Export(index)
	createExportFrame()

	local raid = sRTDB.Boss[index]
	if not raid then return end


	local formatter = formatters[sRTDB.Formatter]
	if not formatter then
		sRT:PrintMessage(("The formatter %s isn't registered."):format(sRTDB.Formatter))
		return
	end
	local text = formatter(raid, sRTDB.Loot, killedBosses)
	if not text then
		sRT:PrintMessage(("The formatter didn't return any text for the raid ID %d."):format(index))
		return
	end

	setText(text)
	setFrameHeader(raid.name .. " " .. date(ifDateFormat, raid.date))
	trackerFrame.trackingId = index
	trackerFrame:Show()
	sRTEditBox:SetFocus()
	sRTEditBox:HighlightText()
end

function createExportFrame()
	if trackerFrame then return end

	trackerFrame = CreateFrame("Frame", "sRTFrame", UIParent)
	trackerFrame:Hide()

	trackerFrame:SetWidth(400)
	trackerFrame:SetHeight(500)

	-- thanks to haste for the style
	trackerFrame:SetBackdrop({
		bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16,
		edgeFile = "", edgeSize = 32,
		insets = {left = 1, right = 1, top = 20, bottom = 1},
	})
	trackerFrame:SetBackdropColor(24/255, 24/255, 24/255)
	trackerFrame:ClearAllPoints()
	trackerFrame:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
	trackerFrame:EnableMouse(true)
	trackerFrame:RegisterForDrag("LeftButton")
	trackerFrame:SetMovable(true)
	trackerFrame:SetResizable(true)
	trackerFrame:SetMinResize(280,300)

	local cheader = trackerFrame:CreateFontString(nil,"OVERLAY")
	cheader:ClearAllPoints()
	cheader:SetWidth(240)
	cheader:SetHeight(15)
	cheader:SetPoint("TOPLEFT", trackerFrame, "TOPLEFT", 7, -14)
	cheader:SetPoint("TOPRIGHT", trackerFrame, "TOPRIGHT", -28, -14)
	cheader:SetFont("Fonts\\FRIZQT__.TTF", 12)
	cheader:SetJustifyH("LEFT")
	setFrameHeader = function(text)
		cheader:SetText(text)
	end
	cheader:SetText("Tracker")
	cheader:SetShadowOffset(.8, -.8)
	cheader:SetShadowColor(0, 0, 0, 1)

	local close = trackerFrame:CreateTexture(nil, "ARTWORK")
	close:SetTexture("Interface\\AddOns\\sRT\\textures\\close")
	close:SetTexCoord(0, .625, 0, .9333)
	close:SetWidth(20)
	close:SetHeight(14)
	close:SetPoint("TOPRIGHT", trackerFrame, "TOPRIGHT", -7, -15)

	local closebutton = CreateFrame("Button", nil)
	closebutton:SetParent(trackerFrame)
	closebutton:SetWidth(20)
	closebutton:SetHeight(14)
	closebutton:SetPoint("CENTER", close, "CENTER")
	closebutton:SetScript("OnClick", function()
		trackerFrame:Hide()
		trackerFrame.trackingId = nil
	end)

	local scroll = CreateFrame("ScrollFrame", "sRTScrollFrame", trackerFrame, "UIPanelScrollFrameTemplate")
	scroll:SetPoint("TOPLEFT", trackerFrame, "TOPLEFT", 20, -40)
	scroll:SetPoint("RIGHT", trackerFrame, "RIGHT", -40, 0)
	scroll:SetPoint("BOTTOM", trackerFrame, 0, 20)

	local textField = CreateFrame("EditBox", "sRTEditBox", scroll)
	textField:SetFontObject(ChatFontNormal)
	textField:SetMultiLine(true)
	textField:SetWidth(240)
	textField:SetScript("OnEscapePressed", function() this:ClearFocus() end)
	textField:SetScript("OnTextChanged", function() scroll:UpdateScrollChildRect() end)
	textField:SetText("")
	setText = function(text)
		textField:SetText(text)
	end
	textField:SetAutoFocus(false)
	scroll:SetScrollChild(textField)


end


-- Default formatter
sRT:RegisterOutputFormatter("Plain text", function(attendance, loot, killedBosses)
	local text = attendance.name .. " (" .. attendance.zone .. ") - " .. date(ifDateFormat, attendance.date)
	text = text .. "\n\nPlayers:\n"

	text = text .. table.concat(attendance.attendants, "\n")
	text = text .. "\n\nLoot:\n"

	local lootByTracked = {}
	for i, v in next, loot do
		if not lootByTracked[v.from] then lootByTracked[v.from] = {} end
		table.insert(lootByTracked[v.from], v)
	end
	for k, v in pairs(lootByTracked) do
		if k == attendance.name then
			for i, item in next, v do
				local itemName, _, itemRarity = GetItemInfo(item.item)
				text = text ..  itemName .. " - " .. item.player .. "\n"
			end
		end
	end
	return text
end)

sRT:RegisterOutputFormatter("Inner Focus", function(attendance, loot, killedBosses)
	local dkpString = "<raid>\n"
	local bossKilled = "  <boss>\n    <name>" .. attendance.name .. "</name>\n    <zone>" .. attendance.zone .. "</zone>\n    <date>" .. date(ifDateFormat, attendance.date ) .. "</date>\n"	
	local playerList = "    <players>\n"

	for i, v in next, attendance.attendants do
		playerList = playerList .. "      <player>" .. v .. "</player>\n"
	end
	-- Loot Start
	local lootList = "    <loots>\n"
	local lootByTracked = {}
	for i, v in next, loot do
		if not lootByTracked[v.from] then lootByTracked[v.from] = {} end
		table.insert(lootByTracked[v.from], v)
	end

	for k, v in pairs(lootByTracked) do
		if k == attendance.name then
			lootList = lootList .. "      <loot source=\"" .. k .. "\">"
			for i, item in next, v do
				local itemName, _, itemRarity = GetItemInfo(item.item)
				local id = item.itemId or tonumber(item.item:match("^|%x+|Hitem:(%d+):"))
				local note = item.note or ""
				lootList = lootList .. "\n        <item date=\"" .. date(ifDateFormat, item.date) .. "\" player=\"" .. item.player .. "\" name=\"" .. (itemName or "?") .. "\" quality=\"" .. (itemRarity or "?") .. "\" id=\"" .. id .. "\" note=\"" .. note .. "\" />"
			end
		lootList = lootList .. "\n      </loot>\n"
		end
	end
	lootList = lootList .. "    </loots>\n"
	-- Loot End

	playerList = playerList .. "    </players>\n"
	dkpString = dkpString .. bossKilled .. playerList .. lootList

	dkpString = dkpString .. "  </boss>\n</raid>"
	return dkpString
end)

local ctrtDateFormat = "%m/%d/%y %H:%M:%S"
sRT:RegisterOutputFormatter("CT Raidtracker XML", function(attendance, loot, killedBosses)
	local lootByTracked = {}
	for i, v in next, loot do
		if not lootByTracked[v.from] then lootByTracked[v.from] = {} end
		table.insert(lootByTracked[v.from], v)
	end

	local text = "<RaidInfo>"
	local players = ""

	local fdate = date(ctrtDateFormat, attendance.date)

	text = text .. "<key>" .. fdate .. "</key>"
	text = text .. "<start>" .. fdate .. "</start>"
	text = text .. "<end>" .. fdate .. "</end>"
	text = text .. "<zone>" .. attendance.zone .. "</zone>"

	text = text .. "<PlayerInfos>"

	for i, v in next, attendance.attendants do
		text = text .. "<key" .. i .. "><name>" .. v .. "</name></key" .. i .. ">"
		players = players .. "<key" .. i .. "><player>" .. v .. "</player><time>" .. fdate .. "</time></key" .. i .. ">"
	end

	text = text .. "</PlayerInfos>"

	text = text .. "<BossKills>"
	for i, v in next, killedBosses do
		text = text .. "<key" .. i .. "><name>" .. v .. "</name><time>" .. fdate .. "</time><attendees/></key" .. i .. ">"
	end
	text = text .. "</BossKills>"

	-- Add in attendance.note here somewhere?
	text = text .. "<note><![CDATA[" .. attendance.zone .. " - " .. attendance.name .. " - " .. (attendance.note or "") .. "]]></note>"
	text = text .. "<Join>" .. players .. "</Join>"
	text = text .. "<Leave>" .. players .. "</Leave>"

	text = text .. "<Loot>"

	for k, v in pairs(lootByTracked) do
		if k == attendance.name then
			for i, item in next, v do
				local itemName = GetItemInfo(item.item)
				local id = item.item:match("^|%x+|Hitem:(.*)")
				id = id:sub(1, id:find("|") - 1)

				local color = string.sub(item.item:match("^|(%x+)|"), 2)
				local note = item.note or ""

				text = text .. "<key" .. i .. ">"
				text = text .. "<ItemName>" .. (itemName or "?") .. "</ItemName>"
				text = text .. "<ItemID>" .. id .. "</ItemID>"
				text = text .. "<Icon></Icon>"
				text = text .. "<Class></Class>"
				text = text .. "<SubClass></SubClass>"
				text = text .. "<Color>" .. color .. "</Color>"
				text = text .. "<Count>1</Count>"
				text = text .. "<Player>" .. item.player .. "</Player>"
				text = text .. "<Time>" ..  date(ctrtDateFormat, item.date) .. "</Time>"
				text = text .. "<Zone></Zone>"
				text = text .. "<Boss>" .. item.from .. "</Boss>"
				text = text .. "<Note>" .. note .. "</Note>"
				text = text .. "</key" .. i .. ">"
			end
		end
	end

	text = text .. "</Loot>"
	text = text .. "</RaidInfo>"
	return text
end)

